pub enum AcceptToMempoolError {
MemoryUsageTooHigh,
ConflictingTransaction,
DuplicatedInputs,
Consensus(BlockchainError),
}Expand description
An error returned when we try to add a transaction to the mempool.
Variants
MemoryUsageTooHigh
Memory usage is too high.
ConflictingTransaction
The transaction is conflicting with another transaction in the mempool.
DuplicatedInputs
This transaction has duplicated inputs
Consensus(BlockchainError)
A validation error happened while consensus checking a transaction