Struct MempoolTransaction

struct MempoolTransaction {
    transaction: Transaction,
    time: Instant,
    depends: Vec<u64>,
    children: Vec<u64>,
}
A transaction in the mempool.

This struct holds the transaction itself, the time when we added it to the mempool, the transactions that depend on it, and the transactions that it depends on. We need that extra information to make decisions when to include or not a transaction in mempool or in a block.

Fields

transaction: Transactiontime: Instantdepends: Vec<u64>children: Vec<u64>