type ShortTxid = u64;Expand description
A short transaction id that we use to identify transactions in the mempool.
We use this to keep track of dependencies between transactions, since keeping the full txid would be too expensive. This value is computed using a keyed hash function, with a local key that only we know. This way, peers can’t cause collisions and make our mempool slow.